%Name of the model:             CIDosis-ODE-model-01
%Reference to the web page:     https://2020.igem.org/Team:UCopenhagen/Model
%Contact information:           2vitzemanek@gmail.com
%Date of creation:              27-Oct-2020
%Terms of distribution:         Creative Commons Attribution 4.0 International license


%Description:
%CIDosis-ODE-model-01 is an ODE-based model of an engineered signaling pathway
%in genetically modified yeast Saccharomyces cerevisiae for responding to 
%interleukin (IL) concentrations by synthesizing luciferase enzymes. Its 
%purpose is to provide insights into mechanics of signal transduction and 
%aim design decisions. The pathway starts with associative binding of IL, 
%receptor and coreceptor which leads to release of a transcription factor 
%that initiates transcription followed by translation.
%The receptor and the coreceptor are
%each part of a different engineered transmembrane protein, and their
%extracellular association triggered by IL leads to intracellular association
%which prompts reconstitution of a split ubiquitin. Ubiquitin is cleaved by
%deubiquitinase and the transcription factor that was part of one of the
%transmembrane proteins is released.


%MIRIAM quality checklist:
%-----------------------------------------
%Standardized format:               yes
%Compliance with the standard:		yes
%Single description:                yes
%Model reflects the description:    yes
%Reproducible output:               yes
%-----------------------------------------


%How to read this model:
%Each euquation is a differential equation that characterizes change in concentration of a single molecular species.
%Concentrations (including initial conditions) are given in uM, except for a specific part of the model that is in nM.
%Each parameter is accompanied by a line explaining what is its biological meaning.
%Also, every equation has a comment that specifies the molecular species.

%==========================================================================


%declare variables for reaction rates
global k1 k2 k3 k4 k5 k6 k8 k9 k11 k12 k13 k14 k15;

%ASSIGN VALUES TO THE PARAMETERS:

%binding of IL-1 and IL-1R (or IL-6 and IL-6R, IL-10 and IL-10R1)
k1=18e6*0.000001;

%unbinding of IL-1 and IL-1R  (or IL-6 and IL-6R, IL-10 and IL-10R1)
k2=1.8e-3;

%binding of IL-1+IL-1R and IL-1RAcP
%(or IL-6+IL-6R and gp130, IL-10+IL-10R1 and IL-10R2)
k3=6.3e3*0.000001;

%unbinding of IL-1+IL-1R and IL-1RAcP
%(or IL-6+IL-6R and gp130, IL-10+IL-10R1 and IL-10R2)
k4=0.015;

%association of split ubiquitin
k5=1.8e3*0.000001;

%DUB concentration times DUB activity
k6=1*0.05;

%TF degradation
k8=0.0001925;

%transcription rate divided by gene length, times gene concentration
k9=(5/800)*1;

%mRNA degradation
k11=0.00056;

%translation rate divided by gene length
k12=6.6/516;

%luciferase degradation
k13=0.0001925;

%split-ubiquitin dissociation
k14=1.3e-2;

%k15 is dissociation constant Kd for promoter and TF
k15 = 0.001;


%initial concentrations
%c0=[0.001,1,1,0,0,0,0,0,0];

%final time
tend=10000;

%different concentration of the biomarker (IL-1)
il_concentrations = [10, 1, 0.1, 0.05, 0.01, 0.005, 0.001, 0.0005, 0.0001, 0.00005, 0.00001, 0.000005, 0.000001, 0.0000005, 0.0000001, 0.00000005, 0.00000001, 0.000000005, 0.000000001, 0.0000000005, 0.0000000001, 0.00000000005, 0.00000000001, 0.000000000005, 0.000000000001, 0.0000000000001, 0.00000000000001];

%call function that does simulations and store all data in 'c'
[t,c] = simulate(il_concentrations, tend);

%array with luciferase concentrations at the end of the simulation
output_concentrations = zeros(1, length(c));
for i = 1:length(c)
    output_concentrations(i) = c{i}(end,9);
end

%fraction of the initial biomarker concentration that was used in the simulation
il_used = zeros(1, length(c));
for i = 1:length(c)
    il_used(i) = c{i}(1,1) - c{i}(end,1);
end


output_percent=output_concentrations/output_concentrations(1)*100;
semilogx(il_concentrations,output_percent);
title('Dynamic range of the design #1'); xlabel('IL concentration (in uM)'); ylabel('NanoLuc concentration (% of max)');


%our model is defined here
function dc=chain1ddt(t,c)

%new function, so we declare kinetic parameters again
global k1 k2 k3 k4 k5 k6 k8 k9 k11 k12 k13 k14 k15;

%concentration: c
%kinetic parameter: k, p
%differential equation: d

%IL-1
d1 = - c(1)*c(2)*k1 + c(4)*k2;

%IL-1R (part of one transmembrane protein)
d2 = - c(1)*c(2)*k1 + c(4)*k2;

%IL-1R AcP (accessory protein, part of another transmembrane protein)
d3 = - c(4)*c(3)*k3 + c(5)*k4;

%complex 1 (IL-1 binds IL-1R)
d4 = c(1)*c(2)*k1 - c(4)*k2 - c(4)*c(3)*k3 + c(5)*k4;

%complex 2a (complex 1 binds IL-1R AcP - extracellular)
d5 = c(4)*c(3)*k3 - c(5)*k4 - c(5)*k5 + c(6)*k14;

%complex 2b (intracellular binding of transmembrane proteins)
d6 = c(5)*k5 - c(6)*k14 - c(6)*k6;

%Ste12a
d7 = c(6)*k6 - c(7)*k8;

%mRNA a
d8 = k9*c(7)/(k15+c(7)) - c(8)*k11;

%luciferase
d9 = c(8)*k12 - c(9)*k13;

%from NanoBit documentation, 1 ng of luciferase per well (0.1 mL) appears good -> concentration of 0.01-10 uM.

%put all results into a vector for an easy flow
dc = [d1;d2;d3;d4;d5;d6;d7;d8;d9];

end


%function that does simulations
function [t,c] = simulate(il_concentrations, Tend)

    %preallocate 'c' to make matlab happy
    c = cell(1,length(il_concentrations));
    c(1,length(il_concentrations)) = {NaN(1, length(il_concentrations))};
    
    t = cell(1,length(il_concentrations));
    t(1,length(il_concentrations)) = {NaN(1, length(il_concentrations))};
    
    %loop that does simulationns for all concentrations we like
    for i = 1:length(il_concentrations)
        
        %array with initial concentrations
        c0=[il_concentrations(i),1,1,0,0,0,0,0,0];
        
        %use solver ODE45 to solve 'chain1ddt' with above initial condition
        [t{i},c{i}]= ode23s(@chain1ddt, [0,Tend], c0);
        
    end

end

