%Name of the model:             CIDosis-ODE-model-03
%Reference to the web page:     https://2020.igem.org/Team:UCopenhagen/Model
%Contact information:           2vitzemanek@gmail.com
%Date of creation:              27-Oct-2020
%Terms of distribution:         Creative Commons Attribution 4.0 International license


%Description:
%CIDosis-ODE-model-03 is an ODE-based model of an engineered signaling pathway
%in genetically modified yeast Saccharomyces cerevisiae for responding to
%interleukin (IL) concentrations by synthesizing luciferase enzymes. Its
%purpose is to provide insights into mechanics of signal transduction and
%aim design decisions. The pathway starts with associative binding of IL,
%receptor and coreceptor and proceeds to activation of a pheromone cascade
%(implemented from www.ebi.ac.uk/biomodels/BIOMD0000000032) which leads to
%activation of a transcription factor that initiates transcription followed
%by translation.
%The receptor and the coreceptor are
%each part of a different engineered transmembrane protein, and their
%extracellular association triggered by IL leads to intracellular association
%which prompts reconstitution of a split TEV protease. Active TEV protease
%cleaves engineered G protein alpha subunit and thereafter G protein beta
%gamma complex initiates the pheromone cascade.


%MIRIAM quality checklist:
%-----------------------------------------
%Standardized format:               yes
%Compliance with the standard:		yes
%Single description:                yes
%Model reflects the description:    yes
%Reproducible output:               yes
%-----------------------------------------


%How to read this model:
%Each euquation is a differential equation that characterizes change in concentration of a single molecular species.
%Concentrations (including initial conditions) are given in uM, except for a specific part of the model that is in nM.
%Each parameter is accompanied by a line explaining what is its biological meaning.
%Also, every equation has a comment that specifies the molecular species.

%==========================================================================


%declare variables for reaction rates
global k1 k2 k3 k4 k5 k6 k8 k9 k11 k12 k13 k14 k15 k17 k18 k19 k20;
global p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 p30 p31 p32 p33 p34 p35;

%ASSIGN VALUES TO THE PARAMETERS:

%binding of IL-1 and IL-1R (or IL-6 and IL-6R, IL-10 and IL-10R1)
k1=18e6*0.000001;

%unbinding of IL-1 and IL-1R  (or IL-6 and IL-6R, IL-10 and IL-10R1)
k2=1.8e-3;

%binding of IL-1+IL-1R and IL-1RAcP
%(or IL-6+IL-6R and gp130, IL-10+IL-10R1 and IL-10R2)
k3=6.3e3*0.000001;

%unbinding of IL-1+IL-1R and IL-1RAcP
%(or IL-6+IL-6R and gp130, IL-10+IL-10R1 and IL-10R2)
k4=0.015;

%association of split TEV protease
k5=1.8e3*0.000001;

%rate of STE7 leading to STE12 activation, times STE12 concentration
k6=2000*0.000001*0.25;

%TF degradation
k8=0.0001925;

%transcription rate divided by gene length, times gene copy number
k9=(5/800)*1;

%mRNA degradation
k11=0.00056;

%translation rate divided by gene length
k12=6.6/516;

%luciferase degradation
k13=0.0001925;

%split-TEV dissociation
k14=1.3e-2;

%k15 is G-protein concentration times TEV-protease activity
k15 = 1*0.000907;
%k15 = 1*0.18;

%k17 is rate of G-beta-gamma leading to STE7 activation, times STE5 concentration
k17 = 0.0013*0.25;

%k18 is G-beta-gamma degradation
k18 = 0.0001925;

%k19 is rate of pheromone pathway protein deactivation
k19 = 0.0002;

%k20 is dissociation constant Kd for promoter and TF
k20 = 0.001;

%parameters inherited from the implemented model of the pheromone cascade
p10=0.1;
p11=5.0;
p12=1.0;
p13=3.0;
p14=1.0;
p15=3.0;
p16=3.0;
p17=100.0;
p18=5.0;
p19=1.0;
p20=10.0;
p21=5.0;
p22=47.0;
p23=5.0;
p24=345.0;
p25=5.0;
p26=50.0;
p27=5.0;
p28=140.0;
p29=10.0;
p30=1.0;
p31=250.0;
p32=5.0;
p33=50.0;
p34=18.0;
p35=0.0003;


%initial concentrations
%c0=[0.0001,1,1,0,0,0,0,236,0,158,158,106,36,686,78,1000,0,0,0,0,0,0,0,0,200,0,0,0];

%final time
tend=10000;

%different concentration of the biomarker (IL-1)
il_concentrations = [10, 1, 0.1, 0.05, 0.01, 0.005, 0.001, 0.0005, 0.0001, 0.00005, 0.00001, 0.000005, 0.000001, 0.0000005, 0.0000001, 0.00000005, 0.00000001, 0.000000005, 0.000000001, 0.0000000005, 0.0000000001, 0.00000000005, 0.00000000001, 0.000000000005, 0.000000000001, 0.0000000000001, 0.00000000000001];


%call function that does simulations and store all data in 'c'
[t,c] = simulate(il_concentrations, tend);

%array with luciferase concentrations at the end of the simulation
output_concentrations = zeros(1, length(c));
for i = 1:length(c)
    output_concentrations(i) = c{i}(end,28);
end

%fraction of the initial biomarker concentration that was used in the simulation
il_used = zeros(1, length(c));
for i = 1:length(c)
    il_used(i) = c{i}(1,1) - c{i}(end,1);
end


output_percent=output_concentrations/output_concentrations(1)*100;
semilogx(il_concentrations,output_percent);
title('Dynamic range of the design #3'); xlabel('IL concentration (in uM)'); ylabel('NanoLuc concentration (% of max)');ylim([0 100]);


%our model is defined here
function dc=chain1ddt(t,c)

%new function, so we declare kinetic parameters again
global k1 k2 k3 k4 k5 k9 k11 k12 k13 k14 k15 k20;
global p10 p11 p12 p13 p14 p15 p16 p17 p18 p19 p20 p21 p22 p23 p24 p25 p26 p27 p28 p29 p30 p31 p32 p33 p34 p35;

%concentration: c
%kinetic parameter: k, p
%differential equation: d

%IL-1
d1 = - c(1)*c(2)*k1 + c(4)*k2;

%IL-1R (part of one transmembrane protein)
d2 = - c(1)*c(2)*k1 + c(4)*k2;

%IL-1R AcP (accessory protein, part of another transmembrane protein)
d3 = - c(4)*c(3)*k3 + c(5)*k4;

%complex 1 (IL-1 binds IL-1R)
d4 = c(1)*c(2)*k1 - c(4)*k2 - c(4)*c(3)*k3 + c(5)*k4;

%complex 2a (complex 1 binds IL-1R AcP - extracellular)
d5 = c(4)*c(3)*k3 - c(5)*k4 - c(5)*k5 + c(6)*k14;

%complex 2b (intracellular binding of transmembrane proteins)
d6 = c(5)*k5 - c(6)*k14;

%PHEROMONE CASCADE (concentrations in nM):
%--------------------------------------------------------------------------

%G-beta-gamma
d7 = c(6)*1000*k15 - c(7)*c(8)*p10 + c(9)*p11 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;

%complex C
d8 = - c(7)*c(8)*p10 + c(9)*p11 + c(12)*c(15)*p16 - c(8)*p17;
	
%complex D
d9 = c(7)*c(8)*p10 - c(9)*p11 - c(9)*c(16)*p18 + c(17)*p19;
	
%Ste5
d10 = - c(10)*c(11)*p12 + c(12)*p13 + c(8)*p17 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;
	
%Ste11
d11 = - c(10)*c(11)*p12 + c(12)*p13 + c(8)*p17 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;
	
%complex A
d12 = c(10)*c(11)*p12 - c(12)*p13 - c(12)*c(15)*p16;
	
%Ste7
d13 = - c(13)*c(14)*p14 + c(15)*p15 + c(8)*p17 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;
	
%Fus3
d14 = - c(13)*c(14)*p14 + c(15)*p15 + c(8)*p17 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 - c(22)*c(14)*p29 + c(24)*p30 + c(23)*p33;
	
%complex B
d15 = c(13)*c(14)*p14 - c(15)*p15 - c(12)*c(15)*p16;
	
%Ste20
d16 = - c(9)*c(16)*p18 + c(17)*p19 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;
	
%complex E
d17 = c(9)*c(16)*p18 - c(17)*p19 - c(17)*p20 - c(17)*p21;
	
%complex F
d18 = c(17)*p20 - c(18)*p22 - c(18)*p23;
	
%complex G
d19 = c(18)*p22 - c(19)*p24 - c(19)*p25;
	
%complex H
d20 = c(19)*p24 - c(20)*p26 - c(20)*p27;
	
%complex I
d21 = c(20)*p26 - c(21)*p28 + c(24)*p31;
	
%complex L
d22 = c(21)*p28 - c(22)*c(14)*p29 + c(24)*p30 - c(22)*p32;
	
%Fus3PP
d23 = c(21)*p28 - c(23)*p33 - c(25)*c(23)*p34;
	
%complex K
d24 = c(22)*c(14)*p29 - c(24)*p30 - c(24)*p31;
	
%Ste12
d25 = - c(25)*c(23)*p34 + c(26)*p35;
	
%Ste12a
d26 = c(25)*c(23)*p34 - c(26)*p35;

%--------------------------------------------------------------------------

%mRNA
d27 = k9*(c(26)/1000)/(k20+(c(26)/1000)) - c(27)*k11;

%luciferase
d28 = c(27)*k12 - c(28)*k13;

%from NanoBit documentation, 1 ng of luciferase per well (0.1 mL) appears good -> concentration of 0.01-10 uM.

%put all results into a vector for an easy flow
dc = [d1;d2;d3;d4;d5;d6;d7;d8;d9;d10;d11;d12;d13;d14;d15;d16;d17;d18;d19;d20;d21;d22;d23;d24;d25;d26;d27;d28];

end


%function that does simulations
function [t,c] = simulate(il_concentrations, Tend)

    %preallocate 'c' to make matlab happy
    c = cell(1,length(il_concentrations));
    c(1,length(il_concentrations)) = {NaN(1, length(il_concentrations))};
    
    t = cell(1,length(il_concentrations));
    t(1,length(il_concentrations)) = {NaN(1, length(il_concentrations))};
    
    %loop that does simulationns for all concentrations we like
    for i = 1:length(il_concentrations)
        
        %array with initial concentrations
        c0=[il_concentrations(i),1,1,0,0,0,0,236,0,158,158,106,36,686,78,1000,0,0,0,0,0,0,0,0,200,0,0,0];
        
        %use solver ODE45 to solve 'chain1ddt' with above initial condition
        [t{i},c{i}]= ode23s(@chain1ddt, [0,Tend], c0);
        
    end

end
