%Name of the model:             CIDosis-SDE-model-02
%Reference to the web page:     https://2020.igem.org/Team:UCopenhagen/Model
%Contact information:           2vitzemanek@gmail.com
%Date of creation:              27-Oct-2020
%Terms of distribution:         Creative Commons Attribution 4.0 International license


%Description:
%CIDosis-SDE-model-02 is an SDE-based model of an engineered signaling pathway
%in genetically modified yeast Saccharomyces cerevisiae for responding to
%interleukin (IL) concentrations by synthesizing luciferase enzymes. Its
%purpose is to provide insights into mechanics of signal transduction and
%aim design decisions. The pathway starts with associative binding of IL,
%receptor and coreceptor which leads to release of a transcription factor
%that initiates transcription followed by translation.
%The receptor and the coreceptor are
%each part of a different engineered transmembrane protein, and their
%extracellular association triggered by IL leads to intracellular association
%which prompts reconstitution of a split TEV protease. Active TEV protease
%cleaves membrane-bound transcription factor away from the membrane.


%MIRIAM quality checklist:
%-----------------------------------------
%Standardized format:               no
%Compliance with the standard:		yes
%Single description:                yes
%Model reflects the description:    yes
%Reproducible output:               yes
%-----------------------------------------


%How to read this model:
%Each euquation is a differential equation that characterizes change in concentration of a single molecular species.
%Concentrations (including initial conditions) are given in uM, except for a specific part of the model that is in nM.
%Each parameter is accompanied by a line explaining what is its biological meaning.
%Also, every equation has a comment that specifies the molecular species.
%The noise function specifies the Wiener process for each SDE.

%==========================================================================


%ASSIGN VALUES TO THE PARAMETERS:

%binding of IL-1 and IL-1R (or IL-6 and IL-6R, IL-10 and IL-10R1)
k1=18e6*0.000001;

%unbinding of IL-1 and IL-1R  (or IL-6 and IL-6R, IL-10 and IL-10R1)
k2=1.8e-3;

%binding of IL-1+IL-1R and IL-1RAcP
%(or IL-6+IL-6R and gp130, IL-10+IL-10R1 and IL-10R2)
k3=6.3e3*0.000001;

%unbinding of IL-1+IL-1R and IL-1RAcP
%(or IL-6+IL-6R and gp130, IL-10+IL-10R1 and IL-10R2)
k4=0.015;

%association of split TEV protease
k5=1.8e3*0.000001;

%rate of cleaving TF away by TEV protease, times TF concentration
k6=0.18*1;

%TF degradation
k8=0.0001925;

%transcription rate divided by gene length, times gene concentration
k9=(5/800)*1;

%mRNA degradation
k11=0.00056;

%translation rate divided by gene length
k12=6.6/516;

%luciferase degradation
k13=0.0001925;

%split-TEV dissociation
k14=1.3e-2;

%k15 is dissociation constant Kd for promoter and TF
k15 = 0.001;


%equations defining the signaling pathway
f = @(t,c)[

%IL-1
- c(1)*c(2)*k1 + c(4)*k2;

%IL-1R (part of one transmembrane protein)
- c(1)*c(2)*k1 + c(4)*k2;

%IL-1R AcP (accessory protein, part of another transmembrane protein)
- c(4)*c(3)*k3 + c(5)*k4;

%complex 1 (IL-1 binds IL-1R)
c(1)*c(2)*k1 - c(4)*k2 - c(4)*c(3)*k3 + c(5)*k4;

%complex 2a (complex 1 binds IL-1R AcP - extracellular)
c(4)*c(3)*k3 - c(5)*k4 - c(5)*k5 + c(6)*k14;

%complex 2b (intracellular binding of transmembrane proteins)
c(5)*k5 - c(6)*k14 - c(6)*k6;

%Ste12a
c(6)*k6 - c(7)*k8;

%mRNA a
k9*c(7)/(k15+c(7)) - c(8)*k11;

%luciferase
c(8)*k12 - c(9)*k13;

];


%to ensure continuous randomness
s = RandStream('mt19937ar','Seed','shuffle');
RandStream.setGlobalStream(s);

%noise function
g=rand*0.000000004;


%time step
dt = 1e-4;

%final time
tend = 10;

%whole time vector
t = 0:dt:tend;

%initial conditions
c0=[0.001,1,1,0,0,0,0,0,0];



%options for the SDE solver
opts = sdeset('NonNegative','yes','sdetype','ito');

%number of trials
n=5;

%array with all trials
trials = cell(1,n);
    trials(1,n) = {NaN(1, n)};

%run the simulation n times 
for i=1:n
trials{i} = sde_milstein(f,g,t,c0,opts); % Integrate
end

%plot concentrations of NanoLuc over time, from all trials
figure;
hold on
for i=1:n
    plot(0:dt:tend,trials{i}(:,9));
end
title('title'); xlabel('Time (in seconds)'); ylabel('NanoLuc concentration (in uM)');
hold off