%Name of the model:             CIDosis-SDE-model-03
%Reference to the web page:     https://2020.igem.org/Team:UCopenhagen/Model
%Contact information:           2vitzemanek@gmail.com
%Date of creation:              27-Oct-2020
%Terms of distribution:         Creative Commons Attribution 4.0 International license


%Description:
%CIDosis-SDE-model-03 is an SDE-based model of an engineered signaling pathway
%in genetically modified yeast Saccharomyces cerevisiae for responding to
%interleukin (IL) concentrations by synthesizing luciferase enzymes. Its
%purpose is to provide insights into mechanics of signal transduction and
%aim design decisions. The pathway starts with associative binding of IL,
%receptor and coreceptor and proceeds to activation of a pheromone cascade
%(implemented from www.ebi.ac.uk/biomodels/BIOMD0000000032) which leads to
%activation of a transcription factor that initiates transcription followed
%by translation.
%The receptor and the coreceptor are
%each part of a different engineered transmembrane protein, and their
%extracellular association triggered by IL leads to intracellular association
%which prompts reconstitution of a split TEV protease. Active TEV protease
%cleaves engineered G protein alpha subunit and thereafter G protein beta
%gamma complex initiates the pheromone cascade.


%MIRIAM quality checklist:
%-----------------------------------------
%Standardized format:               no
%Compliance with the standard:		yes
%Single description:                yes
%Model reflects the description:    yes
%Reproducible output:               yes
%-----------------------------------------


%How to read this model:
%Each euquation under 'f' is a differential equation that characterizes change in concentration of a single molecular species.
%Concentrations (including initial conditions) are given in uM, except for a specific part of the model that is in nM.
%Each parameter is accompanied by a line explaining what is its biological meaning.
%Also, every equation has a comment that specifies the molecular species.
%The noise function specifies the Wiener process for each SDE.

%==========================================================================


%ASSIGN VALUES TO THE PARAMETERS:

%binding of IL-1 and IL-1R (or IL-6 and IL-6R, IL-10 and IL-10R1)
k1=18e6*0.000001;

%unbinding of IL-1 and IL-1R  (or IL-6 and IL-6R, IL-10 and IL-10R1)
k2=1.8e-3;

%binding of IL-1+IL-1R and IL-1RAcP
%(or IL-6+IL-6R and gp130, IL-10+IL-10R1 and IL-10R2)
k3=6.3e3*0.000001;

%unbinding of IL-1+IL-1R and IL-1RAcP
%(or IL-6+IL-6R and gp130, IL-10+IL-10R1 and IL-10R2)
k4=0.015;

%association of split TEV protease
k5=1.8e3*0.000001;

%rate of STE7 leading to STE12 activation, times STE12 concentration
k6=2000*0.000001*0.25;

%TF degradation
k8=0.0001925;

%transcription rate divided by gene length, times gene copy number
k9=(5/800)*1;

%mRNA degradation
k11=0.00056;

%translation rate divided by gene length
k12=6.6/516;

%luciferase degradation
k13=0.0001925;

%split-TEV dissociation
k14=1.3e-2;

%k15 is G-protein concentration times TEV-protease activity
k15 = 1*0.18;

%k17 is rate of G-beta-gamma leading to STE7 activation, times STE5 concentration
k17 = 0.0013*0.25;

%k18 is G-beta-gamma degradation
k18 = 0.0001925;

%k19 is rate of pheromone pathway protein deactivation
k19 = 0.0002;

%k20 is dissociation constant Kd for promoter and TF
k20 = 0.001;

%parameters inherited from the implemented model of the pheromone cascade
p10=0.1;
p11=5.0;
p12=1.0;
p13=3.0;
p14=1.0;
p15=3.0;
p16=3.0;
p17=100.0;
p18=5.0;
p19=1.0;
p20=10.0;
p21=5.0;
p22=47.0;
p23=5.0;
p24=345.0;
p25=5.0;
p26=50.0;
p27=5.0;
p28=140.0;
p29=10.0;
p30=1.0;
p31=250.0;
p32=5.0;
p33=50.0;
p34=18.0;
p35=0.0003;

%equations defining the signaling pathway
f = @(t,c)[

%IL-1
- c(1)*c(2)*k1 + c(4)*k2;

%IL-1R (part of one transmembrane protein)
- c(1)*c(2)*k1 + c(4)*k2;

%IL-1R AcP (accessory protein, part of another transmembrane protein)
- c(4)*c(3)*k3 + c(5)*k4;

%complex 1 (IL-1 binds IL-1R)
c(1)*c(2)*k1 - c(4)*k2 - c(4)*c(3)*k3 + c(5)*k4;

%complex 2a (complex 1 binds IL-1R AcP - extracellular)
c(4)*c(3)*k3 - c(5)*k4 - c(5)*k5 + c(6)*k14;

%complex 2b (intracellular binding of transmembrane proteins)
c(5)*k5 - c(6)*k14;

%PHEROMONE CASCADE (concentrations in nM):
%--------------------------------------------------------------------------

%G-beta-gamma
c(6)*1000*k15 - c(7)*c(8)*p10 + c(9)*p11 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;

%complex C
- c(7)*c(8)*p10 + c(9)*p11 + c(12)*c(15)*p16 - c(8)*p17;
	
%complex D
c(7)*c(8)*p10 - c(9)*p11 - c(9)*c(16)*p18 + c(17)*p19;
	
%Ste5
- c(10)*c(11)*p12 + c(12)*p13 + c(8)*p17 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;
	
%Ste11
- c(10)*c(11)*p12 + c(12)*p13 + c(8)*p17 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;
	
%complex A
c(10)*c(11)*p12 - c(12)*p13 - c(12)*c(15)*p16;
	
%Ste7
- c(13)*c(14)*p14 + c(15)*p15 + c(8)*p17 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;
	
%Fus3
- c(13)*c(14)*p14 + c(15)*p15 + c(8)*p17 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 - c(22)*c(14)*p29 + c(24)*p30 + c(23)*p33;
	
%complex B
c(13)*c(14)*p14 - c(15)*p15 - c(12)*c(15)*p16;
	
%Ste20
- c(9)*c(16)*p18 + c(17)*p19 + c(17)*p21 + c(18)*p23 + c(19)*p25 + c(20)*p27 + c(22)*p32;
	
%complex E
c(9)*c(16)*p18 - c(17)*p19 - c(17)*p20 - c(17)*p21;
	
%complex F
c(17)*p20 - c(18)*p22 - c(18)*p23;
	
%complex G
c(18)*p22 - c(19)*p24 - c(19)*p25;
	
%complex H
c(19)*p24 - c(20)*p26 - c(20)*p27;
	
%complex I
c(20)*p26 - c(21)*p28 + c(24)*p31;
	
%complex L
c(21)*p28 - c(22)*c(14)*p29 + c(24)*p30 - c(22)*p32;
	
%Fus3PP
c(21)*p28 - c(23)*p33 - c(25)*c(23)*p34;
	
%complex K
c(22)*c(14)*p29 - c(24)*p30 - c(24)*p31;
	
%Ste12
- c(25)*c(23)*p34 + c(26)*p35;
	
%Ste12a
c(25)*c(23)*p34 - c(26)*p35;

%--------------------------------------------------------------------------

%mRNA
k9*(c(26)/1000)/(k20+(c(26)/1000)) - c(27)*k11;

%luciferase
c(27)*k12 - c(28)*k13;

];


%to ensure continuous randomness
s = RandStream('mt19937ar','Seed','shuffle');
RandStream.setGlobalStream(s);

%noise function
g=rand*0.000000004;


%time step
dt = 1e-4;

%final time
tend = 100;

%whole time vector
t = 0:dt:tend;

%initial conditions
c0=[0.0001,1,1,0,0,0,0,236,0,158,158,106,36,686,78,1000,0,0,0,0,0,0,0,0,200,0,0,0];


%options for the SDE solver
opts = sdeset('NonNegative','yes','sdetype','ito');

%number of trials
n=10;

%array with all trials
trials = cell(1,n);
    trials(1,n) = {NaN(1, n)};

%run the simulation n times 
for i=1:n
trials{i} = sde_milstein(f,g,t,c0,opts); % Integrate
end

%plot concentrations of NanoLuc over time, from all trials
figure;
hold on
for i=1:n
    plot(0:dt:tend,trials{i}(:,28));
end
title('title'); xlabel('Time (in seconds)'); ylabel('NanoLuc concentration (in uM)');
hold off